#include "jaeil.H"

/***************************************************/
/**** MCMC application for Missing data         ****/
/***************************************************/
/**** Programmed by : Jaeil                     ****/
/**** Begin Date   : 08.20.08                   ****/
/**** Last Update  : 09.25.08                   ****/
/***************************************************/
int main(int argc, char **argv) {

  int p;
  int nN, nM, nK;
	int poolsize, ncase, ratio;
	
  /* Declare Structure-type DATA */
  FILE *fp;

	/* Data read */  
  if ((argc !=5 )) {
    //fprintf(stderr, "usage: MCMCitempool [data.txt] [nN] [nM] [p] [ksize]\n");
		fprintf(stderr, "usage: MCMCitempool [data.txt] [nsize] [ncase] [ratio]\n");
    system("PAUSE");
    return 1;
  }  

  fp = fopen(argv[1], "r");
  poolsize = atoi(argv[2]);				
	ncase = atoi(argv[3]);
  ratio = atoi(argv[4]);
	
  if(fp == NULL) { 
    fprintf(stderr, "Interactome data do not exist.\n", argv[1]);
    return 1; 
  }
  if(&poolsize == NULL) { 
    fprintf(stderr, "The number of poolsize was not provided.\n");
    return 1; 
  }
  if(&ncase == NULL) { 
    fprintf(stderr, "The number of ncase in each stratum was not provided.\n");
    return 1; 
  }
  if(&ratio == NULL) { 
    fprintf(stderr, "The number of ratio  was not provided.\n");
    return 1; 
  }

	// Read 20000 pool into the pool bank

  fprintf(stderr, "\n*************************************************\n");
  fprintf(stderr, "************  Welcome to Jaeil MCMC     ***********\n");
  fprintf(stderr, "***            Jaeil  09.13.08                  ***\n");
  fprintf(stderr, "*************************************************\n");
  fprintf(stderr, "*************************************************\n");

	/* Number setting*/
	p=2;
	nN=poolsize;
	nM=4;
	nK=4;
	//nK=pool.maxY+1;
	
	/* Initialize before MH*/
	init_param(&p, &nK, &nN, &nM);
	init_prior(&p, &nK, &nN, &nM);	
	init_trparam(&p, &nK, &nN, &nM);
	init_values(&p, &nK, &nN, &nM);
	init_DATA(&nN, &nM, &p); 

	/* True beta1 Beta2 beta1<- 0.5; beta2<- 0.05;*/
	read_data(fp, &nN, &nM, &p);

	//Doing gibbs
	mhgibbs(1);
		
	// Check the first chain converge or not
	res_print(1);

  /* Relase memeroy*/
	free_param(&nN) ;
	free_prior();
	free_trparam(&nN, &nM);
  free_data(&nN, &p);  
	fclose(fp);
  return 0;
}

void res_print(int niter)
{

  int i;
  /* Declare Structure-type DATA */
  FILE  *fpY;
	double E_dev=0.0, DEV=0.0;

	char filename[30];
	char buffer[10];

	for(i=0; i<MAX_ITER-BURN;i++)
	{
		E_dev+=trparam.DIC[i];
	}
	E_dev = E_dev/(double)(i-1);

 	param.beta1[0]=vec_mean2(trparam.beta1[0], BURN, MAX_ITER-BURN);
	param.beta1[1]=vec_mean2(trparam.beta1[1], BURN, MAX_ITER-BURN);
	param.beta1[2]=vec_mean2(trparam.beta1[2], BURN, MAX_ITER-BURN);
	param.beta1[3]=vec_mean2(trparam.beta1[3], BURN, MAX_ITER-BURN);

	param.beta2[0] = vec_mean2(trparam.beta2[0], BURN, MAX_ITER-BURN);
	param.beta2[1] = vec_mean2(trparam.beta2[1], BURN, MAX_ITER-BURN);

	param.phi[0]=0;
	param.phi[3]=1;
	//param.phi[pool.maxY]=1;
	param.phi[1] = vec_mean2(trparam.phi[1], BURN, MAX_ITER-BURN);
	param.phi[2] = vec_mean2(trparam.phi[2], BURN, MAX_ITER-BURN);
	param.phi[3] = vec_mean2(trparam.phi[3], BURN, MAX_ITER-BURN);

	DEV=Calc_DIC(0);

	strcpy(filename, "\chain");
	itoa(niter, buffer, 10);
	strcat(filename, buffer);
	strcat(filename, ".txt");

	fprintf(stderr,"\nOutput summary..");
	/* Result Saving Step*/
	fpY=fopen(filename, "w");
  
	fprintf(fpY, "Beta1 \t Beta2 \t phi1 \t phi2 \t phi3 \t DIC \t AIC \n");
  for(i=BURN;i<MAX_ITER;i++) 
	fprintf(fpY, "%f \t %f \t %f \t %f \t %f \t %f \t %f \n", trparam.beta2[0][i], trparam.beta2[1][i], trparam.phi[1][i], trparam.phi[2][i], trparam.phi[3][i], 2*E_dev-DEV, DEV+2*7.0);

  fprintf(stderr, "..\nThank you for using Jaeil!\n");
	fclose(fpY);
}

