#include "jaeil.H"

/****************************************************************************/
/**************************** likelihood.c ***********************************/
/****************************************************************************/

/* Temprarily save the current value to param*/
void value_trans(int nstep)
{
  int i;
  for(i=0;i<data.nM;i++) param.beta1[i] = trparam.beta1[i][nstep];
  for(i=0;i<param.np;i++) param.beta2[i] =trparam.beta2[i][nstep];
	for(i=0;i<param.ksize;i++) param.phi[i] = trparam.phi[i][nstep];
}


void setPhi()
{
	int i;
	for(i=1;i<param.ksize;i++)
		param.phi[i]= param.phi[i-1]+param.gamma[i];
}

double arms_beta1(double x, int nopt)
{
	int i, j;
	double tmp    = 0.0;
	double tmpNum1 = 0.0;
	double tmpDen1 = 0.0;
	
	param.beta1[nopt]=x;
  for(i=0;i<data.ntot;i++) 
	{	
		if(data.Y[i]==nopt) tmp += 1.0;
	}
  tmpNum1 =  - 1/(2*pow(prior.Vbeta1,2))*pow(param.beta1[nopt] - prior.Mbeta1[nopt] - tmp*pow(prior.Vbeta1,2) ,2);
  tmp=0.0;
  tmpDen1=0;
  for(i=0;i<data.ntot;i++)
  {
		tmp=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen1+=log(tmp);
  }

	return ( tmpNum1 - tmpDen1 );
}

double arms_beta2(double x, int nopt)
{
	int i, j;
	double tmp     = 0.0;
	double tmpNum1 = 0.0;
	double tmpDen1 = 0.0;
	
	param.beta2[nopt]=x;
  for(i=0;i<data.ntot;i++) 
	{	
		tmp+=param.phi[data.Y[i]]*data.X[i][nopt];
	}
  tmpNum1 =  - 1/(2*pow(prior.Vbeta2,2))*pow(param.beta2[nopt] - prior.Mbeta2[nopt] - tmp*pow(prior.Vbeta2,2) ,2);
  tmp=0.0;
  tmpDen1=0;
  for(i=0;i<data.ntot;i++)
  {
		tmp=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen1+=log(tmp);
  }

	return (tmpNum1 - tmpDen1 );
}

double arms_Gamma(double x, int nopt)
{
	int i, j;
	double tmp    = 0.0;
	double tmpNum1 = 0.0;
	double tmpDen1 = 0.0;
	
	param.phi[nopt] =x;
  for(i=0;i<data.ntot;i++) 
	{	
		if(data.Y[i]==nopt) tmp += vec2_sum(param.beta2, data.X[i], param.np);
	}
  tmpNum1 =  - 1/(2*pow(prior.VPhi ,2))*pow(param.phi[nopt] - prior.MPhi[nopt] - tmp*pow(prior.VPhi ,2) ,2);
  tmp=0.0;
  tmpDen1=0;
  for(i=0;i<data.ntot;i++)
  {
		tmp=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen1+=log(tmp);
  }
	return ( tmpNum1 - tmpDen1 );
}

double calcGammaMH(int nstep)
{
	int i, j;
	double tmp    = 0.0;
	double tmp2		= 0.0;
	double acceptProb = 0.0;
	double tmpNum1 = 0.0;
	double tmpDen1 = 0.0;
	double tmpNum2 = 0.0;
	double tmpDen2 = 0.0;
	double tmpBuf=0.0;

	/* Just one call to set param values from param_history */
	tmp=0.0; tmp2=0.0;	tmpDen1=0;
	for(i=0;i<data.ntot;i++) 
	{
		tmp += param.phi[data.Y[i]]*(vec2_sum(param.beta2, data.X[i], param.np)) ;
		tmp2=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp2+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen1+=log(tmp2);
	}

	tmpNum1 = tmp;
	// Updating New gamma from 
	rdirichlet();
	setPhi();
	/* Generate New candidate   from proposal? Which Proposal */
	
	tmp=0.0; tmp2=0.0;	tmpDen2=0;
	for(i=0;i<data.ntot;i++) 
	{
		tmp += param.phi[data.Y[i]]*(vec2_sum(param.beta2, data.X[i], param.np)) ;
		tmp2=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp2+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen2+=log(tmp2);
	}
	tmpNum2 = tmp;
	
	/* Accept or not : Update or Not*/
	acceptProb= exp(-tmpNum1 - tmpDen2 + tmpNum2 + tmpDen1 )  ;
	if(acceptProb > 1.0) acceptProb = 1.0;
	if (drand48() < acceptProb)
	{
		for(i=1;i<param.ksize;i++)
		{
			trparam.gamma[i][nstep+1]= param.gamma[i];
			trparam.phi[i][nstep+1]= param.phi[i];
		}
	}else{
		for(i=1;i<param.ksize;i++)
		{
			/* Restore previous value in case of rejection */
			trparam.gamma[i][nstep+1]= trparam.gamma[i][nstep];
			trparam.phi[i][nstep+1]= trparam.phi[i][nstep];
			param.gamma[i]= trparam.gamma[i][nstep+1];
			param.phi[i]= trparam.phi[i][nstep+1];
		}
		setPhi();
	}
  return 0;  
}

double Calc_DIC(int nstep)
{
	int i, j;
	double tmp    = 0.0;
	double tmpNum1 = 0.0;
	double tmpDen1 = 0.0;
	tmp=0;
	for(i=0;i<data.ntot;i++) 
	{
		tmp += param.phi[data.Y[i]]*vec2_sum(param.beta2, data.X[i], param.np) + param.beta1[data.Y[i]] ;
	}
	tmpNum1 = tmp;

	tmp=0.0;
	tmpDen1=0;
  for(i=0;i<data.ntot;i++)
  {
		tmp=0.0;
    for(j=0;j<data.nM;j++) 
		{	
			tmp+= exp(param.beta1[j]+param.phi[j]*(vec2_sum(param.beta2, data.X[i], param.np) ));
		}
    tmpDen1+=log(tmp);
  }

	return 2*(tmpDen1-tmpNum1);
}

/**************************************/
/*** Metropolis-Hastings with Gibbs ***/
/**************************************/
void mhgibbs(int nsimul)
{
	//initial setting for ARMS
  int err, ninit = 4, npoint = 100, nsamp = 1, ncent = 4 ;
  int neval;
  double xinit[10]={-1.0,-0.5, 0.5,1.0}, xl = -7.0, xr = 7.0;
	double yl=-4.5, yr=4.5;
	double zl=-15.5, zr=15.5;
  double xsamp[100], xcent[10], qcent[10] = {5., 30., 70., 95.};
  unsigned seed = 44;
  double convex = 1.0;
	double convex2 =0.0;
  int dometrop = 0;			// We know log-concavity
  double xprev = 0.0;

  int j, k;
  
  /* MH-Gibbs Iteration Start*/
  for(k=0;k<MAX_ITER-1;k++)
  {
		value_trans(k);
		
		printf("%dth :Iteraion # %d \n" ,nsimul , k);		
		
		for(j=1;j<data.nM;j++)
		{
			xprev =trparam.beta1[j][k];
			err=arms(xinit,ninit,&zl,&zr,arms_beta1 ,&convex2, npoint,dometrop,&xprev,xsamp,nsamp,qcent,xcent,ncent,&neval, j);
			trparam.beta1[j][k+1]=xsamp[0];
			param.beta1[j]=xsamp[0]; 
		}
		for(j=0;j<param.np;j++)
		{
			//xprev=trparam.beta2[j][k];
			xprev=param.beta2[j];
			err=arms(xinit,ninit,&xl,&xr,arms_beta2 ,&convex2, npoint,dometrop,&xprev,xsamp,nsamp,qcent,xcent,ncent,&neval, j);
			trparam.beta2[j][k+1]=xsamp[0];
			param.beta2[j]=xsamp[0];
		}

		if (nsimul==0)
		{
			/* Gamma*/
			for(j=1;j<param.ksize;j++)
			{
				if(j==param.ksize-1)
				{
					param.phi[j]=1;
					trparam.phi[j][k+1]=1;
				}else{
					xprev =trparam.phi[j][k];
					err=arms(xinit,ninit,&yl,&yr,arms_Gamma ,&convex2, npoint,0,&xprev,xsamp,nsamp,qcent,xcent,ncent,&neval, j);
					trparam.phi[j][k+1]=xsamp[0];
					param.phi[j]=xsamp[0];
				}
			}
		} else
		{
			calcGammaMH(k);
		}
		if(k>=BURN) trparam.DIC[k-BURN] = Calc_DIC(k);

    if(err>0){
      printf("Error Occurs at %d\n", err);
			system("PAUSE");
      exit(1);
    }
  }
}



