#include "jaeil.H"


/****************************************************************************/
/**************************** mMath.c ***********************************/
/****************************************************************************/

/*  Simple Random Number generator*/
double drand48 (void)
{
   return ((double)rand() + 0.5)/((double)RAND_MAX + 1.0);
}

double rexp (double scale)
{
	double U;
	U = -log (drand48 ()) / scale;
	return U;
}

double rnormal(double mu, double sigma)
{
	static int iset=0;
	static double gset;
	double fac,rsq,v1,v2;
   
	if  (iset == 0) {
		do {
			v1=2.0*drand48()-1.0;
			v2=2.0*drand48()-1.0;
			rsq=v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=mu + sigma*v1*fac;
		iset=1;
		return mu + sigma*v2*fac;
	} else {
		iset=0;
		return gset;
	}
}
	    		 
double rnorm (double mu, double sigma)
{
  double y1, y2, u1, u2, s, b, x;
  s = 1.0;
  
  while (s >= 1) { 
    y1 = drand48 () ;
    y2 = drand48 () ;
    u1 = 2*y1 - 1;
    u2 = 2*y2 - 1;
    
    s = pow (u1,2) + pow (u2,2);
        if (s >= 1) continue;
    b = sqrt(( - (2 * log(s) / s)));
    
    x = b * u1;
    x = mu + sigma * x;
  }
	return x;
}

double rgamma (double shape, double scale)
{
   double  b, h, r, g, f, x, r2, d, gamma1=10;

   b = shape - 1.0;
   if (b >=0) {
	   h = sqrt (3 * shape - 0.75);
       do {
          do {
             do {
                r = drand48 ();
                g = r - pow(r, 2);
             } while (g <= 0.0);
             f = (r - 0.5) * h /sqrt(g);
   			 x = b + f;
           } while (x <= 0.0);
   			r2 = drand48 ();
   			d = 64 * g * (pow(r2*g, 2));
   			if (d <= 0) {
               gamma1 = x;
               break;
            }
   			if (d*x < (x - 2*f*f)) {
                gamma1 = x;
                 break;
            }
       } while (log(d) > 2*(b*log(x/b) - f));
       gamma1 = x;
       gamma1 = (1 / scale) * gamma1;
   }
   else if (b < 0) {
	   x = rgamma (shape+1, 1);
	   r = drand48 ();
	   x = x*pow(r, 1/shape);
	   gamma1 = x / scale;
   }
   return gamma1;
}     	

//rdirichlet(param.gamma, prior.Gprob, param.ksize);
//void rdirichlet(double *val, double *prob, int len)
void rdirichlet()
{
	int i;
	double sum =0.0;
	for(i=1;i<param.ksize;i++)
	{
		param.gamma[i]= rgamma(prior.Gprob[i],1);
	}
	sum=vec_sum(param.gamma, param.ksize);

	for(i=0;i<param.ksize;i++) param.gamma[i] = param.gamma[i]/sum;
}
/*
void rdirichlet(double *val, double *prob, int len)
{
	int i;
	double sum = 0.0;
	for(i=0; i<len; i++)
	{
		val[i] = rgamma(prob[i], 1);
	}
	sum = vec_sum(val, len);
	for(i=0; i<len; i++) val[i] = val[i]/sum;

}
*/
double vec2_sum(const double *vec1, const double *vec2, int len) {
  int i;
  double res;
  res=vec1[0]*vec2[0];
  for(i=1;i<len;i++) res+=vec1[i]*vec2[i];
  return res;
}

double vec_sum(const double *vec, int len) {
  int i;
  double res;
  res=vec[0];
  for(i=1;i<len;i++) res+=vec[i];
  return res;
}

int vec_max(const int *vec, int len) {
  int i;
  int res;
  res=vec[0];
  for(i=1;i<len;i++) {
    if(res<vec[i]) res=vec[i];
  }
  return res;
}

double vec_min(const double *vec, int len) {
  int i;
  double res;
  res=vec[0];
  for(i=1;i<len;i++) {
    if(res>vec[i]) res=vec[i];
  }
  return res;
}

double vec_mean(const double *vec, const int len) {
  double tmp=0.0;
  int i;
  for(i=0;i<len;i++) tmp+=vec[i];
  tmp=tmp/((double) len);
  return tmp;
}

double vec_mean2(const double *vec, int start, int len) {
  double tmp=0.0;
  int i;
  for(i=start;i<(start+len);i++) tmp+=vec[i];
  tmp=tmp/(double)len;
  return tmp;
}

double vec_var(const double *vec, int len) {
  double mean=0.0;
  double var=0.0;
  int i;
  for(i=0;i<len;i++) mean+=vec[i];
  mean=mean/((double) len);
  for(i=0;i<len;i++) var+=pow((vec[i]-mean),2);
  var/=((double) (len-1));
  var=sqrt(var);
  return var;
}




